// -*- C++ -*-
/*!
 * @file  MobileRobotControl.cpp
 * @brief 移動機能を実現するRTコンポーネント(Beego)
 * @date $Date$
 *
 * $Id$
 */

#include "MobileRobotControl.h"

// Module specification
// <rtc-template block="module_spec">
static const char* mobilerobotcontrol_spec[] =
  {
    "implementation_id", "MobileRobotControl",
    "type_name",         "MobileRobotControl",
    "description",       "Locomotion function for Beego",
    "version",           "0.4.0",
    "vendor",            "Kenich Ohara, Osaka Univ.",
    "category",          "Book",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "60.0",
    // Configuration variables
    "conf.default.COMNumber", "COM1",
	"conf.default.limitStraightVel", "50",
	"conf.default.limitRotateVel", "15",
	"conf.default.curveRadius", "100",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
MobileRobotControl::MobileRobotControl(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_odometryOut("Odometry", m_odometry),
    m_LocomotionControlPort("LocomotionControl"),
    m_CorrectionPort("Correction")
    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("Odometry", m_odometryOut);
  
  // Set service provider to Ports
  m_LocomotionControlPort.registerProvider("LControl", "LocomotionControl", m_LControl);
  m_CorrectionPort.registerProvider("LCorrection", "LocomotionCorrection", m_LCorrection);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_LocomotionControlPort);
  registerPort(m_CorrectionPort);

  // </rtc-template>
}

/*!
 * @brief destructor
 */
MobileRobotControl::~MobileRobotControl()
{
	//if(_robot!=0){ delete _robot; }
}



RTC::ReturnCode_t MobileRobotControl::onInitialize()
{
  std::cout << "##### MobileRobotControl #####" << std::endl;
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("COMNumber", m_comnumber, "COM1");
  bindParameter("limitStraightVel", m_limitStVel, "50");
  bindParameter("limitRotateVel", m_limitRotVel, "15");
  bindParameter("curveRadius", m_curveRadius, "100");

  // 出力ポートのバッファを確保
  m_odometry.data.length(3);

  // サービスポートにRobotSystem::MobileRobotControlインスタンスをセット
  m_LCorrection.setRobotControl(&_robot);
  m_LControl.setRobotControl(&_robot);

  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t MobileRobotControl::onFinalize()
{
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MobileRobotControl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MobileRobotControl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MobileRobotControl::onActivated(RTC::UniqueId ec_id)
{
	// 座標系の設定
	_robot.setCoordinate(RobotSystem::ROBOT_COORDINATE);

	// ビーゴ接続オプションの設定
	int argc(2);
	std::string cmd;
	if(m_comnumber.find("COM") == std::string::npos){
		cmd = "--simulator";
#ifdef MONITOR
		vmonitor::show();
		vmonitor::start();
#endif
	}else{
		cmd = "--run_port=" + m_comnumber;
	}
	char* argv[2]={"./MobileRobotControlComp.exe", const_cast<char*>(cmd.c_str())};
	_robot.setCmdArg(argc, argv);

	// ビーゴへ接続
	std::cout << "connecting to beego." << std::endl;
	if(!_robot.Connect()){
		return RTC::RTC_ERROR;
	}

	// コンフィグレーションパラメータの設定
	_robot.setVelLimit(m_limitStVel, m_limitRotVel);
	_robot.setCurveRadius(m_curveRadius);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t MobileRobotControl::onDeactivated(RTC::UniqueId ec_id)
{
	// ビーゴから切断
	std::cout << "disconnecting to beego." << std::endl;
	if(!_robot.Disconnect()){
		return RTC::RTC_ERROR;
	}
#ifdef MONITOR
	if(m_comnumber.find("COM") == std::string::npos){
		vmonitor::pause();
		vmonitor::hide();
		vmonitor::clear(3);
	}
#endif
	
  return RTC::RTC_OK;
}



RTC::ReturnCode_t MobileRobotControl::onExecute(RTC::UniqueId ec_id)
{
	// 自己位置を取得
	std::cout << "getting current position." << std::endl;
	_robot.getLocalPosition((int&)m_odometry.data[0], (int&)m_odometry.data[1], (int&)m_odometry.data[2]);
	std::cout << "x coordinate : " << m_odometry.data[0] << "[mm]" << std::endl;
	std::cout << "y coordinate : " << m_odometry.data[1] << "[mm]" << std::endl;
	std::cout << "pose angle : " << m_odometry.data[2] << "[deg]" << std::endl;
	m_odometryOut.write();

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MobileRobotControl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t MobileRobotControl::onError(RTC::UniqueId ec_id)
{
	std::cout << "Please connect Beego. \r";
	
  return RTC::RTC_OK;
}



RTC::ReturnCode_t MobileRobotControl::onReset(RTC::UniqueId ec_id)
{
	std::cout << std::endl;
#ifdef MONITOR
	if(m_comnumber.find("COM") == std::string::npos){
		vmonitor::pause();
		vmonitor::hide();
	}
#endif
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t MobileRobotControl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t MobileRobotControl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void MobileRobotControlInit(RTC::Manager* manager)
  {
    RTC::Properties profile(mobilerobotcontrol_spec);
    manager->registerFactory(profile,
                             RTC::Create<MobileRobotControl>,
                             RTC::Delete<MobileRobotControl>);
  }
  
};


